@@ -364,6 +364,20 @@ static inline void fd_lock_tgid(int fd, uint desired_tgid)
 	}
 }
 
+/*
+ * Try to lock the tgid, keeping the current tgid value.
+ * Returns 1 on success, or 0 on failure.
+ */
+static inline int fd_lock_tgid_cur(int fd)
+{
+	uint old = _HA_ATOMIC_LOAD(&fdtab[fd].refc_tgid) & 0x7fff;
+
+	if (_HA_ATOMIC_CAS(&fdtab[fd].refc_tgid, &old, (old | 0x8000) + 0x10000))
+		return 1;
+	return 0;
+}
+
+
 /* Grab a reference to the FD's TGID, and return the tgid. Note that a TGID of
  * zero indicates the FD was closed, thus also fails (i.e. no need to drop it).
  * On non-zero (success), the caller must release it using fd_drop_tgid().
@@ -373,6 +387,10 @@ static inline uint fd_take_tgid(int fd)
 	uint old;
 
 	old = _HA_ATOMIC_FETCH_ADD(&fdtab[fd].refc_tgid, 0x10000) & 0xffff;
+	while (old & 0x8000) {
+                old = _HA_ATOMIC_LOAD(&fdtab[fd].refc_tgid) & 0xffff;
+                __ha_cpu_relax();
+	}
 	if (likely(old))
 		return old;
 	HA_ATOMIC_SUB(&fdtab[fd].refc_tgid, 0x10000);
@@ -398,6 +416,11 @@ static inline uint fd_grab_tgid(int fd, uint desired_tgid)
 	uint old;
 
 	old = _HA_ATOMIC_FETCH_ADD(&fdtab[fd].refc_tgid, 0x10000) & 0xffff;
+	/* If the tgid is locked, wait until it no longer is */
+	while (old & 0x8000) {
+		old = _HA_ATOMIC_LOAD(&fdtab[fd].refc_tgid) & 0xffff;
+		__ha_cpu_relax();
+	}
 	if (likely(old == desired_tgid))
 		return 1;
 	HA_ATOMIC_SUB(&fdtab[fd].refc_tgid, 0x10000);
@@ -425,6 +448,22 @@ static inline void fd_claim_tgid(int fd, uint desired_tgid)
 	}
 }
 
+/*
+ * Update the FD's TGID.
+ * This should be called with the lock held, and will drop the lock once
+ * the TGID is updated.
+ * The reference counter is however preserved.
+ */
+static inline void fd_update_tgid(int fd, uint desired_tgid)
+{
+	unsigned int orig_tgid = fdtab[fd].refc_tgid;
+	unsigned int new_tgid;
+	/* Remove the lock, and switch to the new tgid */
+	do {
+		new_tgid = (orig_tgid & 0xffff0000) | desired_tgid;
+	} while (!_HA_ATOMIC_CAS(&fdtab[fd].refc_tgid, &orig_tgid, new_tgid) && __ha_cpu_relax());
+}
+
 /* atomically read the running mask if the tgid matches, or returns zero if it
  * does not match. This is meant for use in code paths where the bit is expected
  * to be present and will be sufficient to protect against a short-term group
