@@ -48,29 +48,25 @@
 #define H3_SETTINGS_MAX_FIELD_SECTION_SIZE   0x06
 #define H3_SETTINGS_QPACK_BLOCKED_STREAMS    0x07
 
-/* Errors. */
+/* RFC 9114 8. Error Handling */
 enum h3_err {
-	H3_NO_ERROR                = 0x100,
-	H3_GENERAL_PROTOCOL_ERROR  = 0x101,
-	H3_INTERNAL_ERROR          = 0x102,
-	H3_STREAM_CREATION_ERROR   = 0x103,
-	H3_CLOSED_CRITICAL_STREAM  = 0x104,
-	H3_FRAME_UNEXPECTED        = 0x105,
-	H3_FRAME_ERROR             = 0x106,
-	H3_EXCESSIVE_LOAD          = 0x107,
-	H3_ID_ERROR                = 0x108,
-	H3_SETTINGS_ERROR          = 0x109,
-	H3_MISSING_SETTINGS        = 0x10a,
-	H3_REQUEST_REJECTED        = 0x10b,
-	H3_REQUEST_CANCELLED       = 0x10c,
-	H3_REQUEST_INCOMPLETE      = 0x10d,
-	H3_MESSAGE_ERROR           = 0x10e,
-	H3_CONNECT_ERROR           = 0x10f,
-	H3_VERSION_FALLBACK        = 0x110,
-
-	QPACK_DECOMPRESSION_FAILED = 0x200,
-	QPACK_ENCODER_STREAM_ERROR = 0x201,
-	QPACK_DECODER_STREAM_ERROR = 0x202,
+	H3_ERR_NO_ERROR                = 0x100,
+	H3_ERR_GENERAL_PROTOCOL_ERROR  = 0x101,
+	H3_ERR_INTERNAL_ERROR          = 0x102,
+	H3_ERR_STREAM_CREATION_ERROR   = 0x103,
+	H3_ERR_CLOSED_CRITICAL_STREAM  = 0x104,
+	H3_ERR_FRAME_UNEXPECTED        = 0x105,
+	H3_ERR_FRAME_ERROR             = 0x106,
+	H3_ERR_EXCESSIVE_LOAD          = 0x107,
+	H3_ERR_ID_ERROR                = 0x108,
+	H3_ERR_SETTINGS_ERROR          = 0x109,
+	H3_ERR_MISSING_SETTINGS        = 0x10a,
+	H3_ERR_REQUEST_REJECTED        = 0x10b,
+	H3_ERR_REQUEST_CANCELLED       = 0x10c,
+	H3_ERR_REQUEST_INCOMPLETE      = 0x10d,
+	H3_ERR_MESSAGE_ERROR           = 0x10e,
+	H3_ERR_CONNECT_ERROR           = 0x10f,
+	H3_ERR_VERSION_FALLBACK        = 0x110,
 };
 
 /* Frame types. */
