@@ -168,7 +168,7 @@ enum {
 	REDIRECT_FLAG_FROM_REQ = 4,     /* redirect rule on the request path */
 	REDIRECT_FLAG_IGNORE_EMPTY = 8, /* silently ignore empty location expressions */
 	REDIRECT_FLAG_KEEP_QS = 16,	/* append the query string to location, if any */
-	REDIRECT_FLAG_COOKIE_FMT = 32,  /* The cookie value is a log-format stirng*/
+	REDIRECT_FLAG_COOKIE_FMT = 32,  /* The cookie value is a log-format string */
 };
 
 /* Redirect types (location, prefix, extended ) */
@@ -190,6 +190,7 @@ enum {
 enum rule_result {
 	HTTP_RULE_RES_CONT = 0,  /* nothing special, continue rules evaluation */
 	HTTP_RULE_RES_YIELD,     /* call me later because some data is missing. */
+	HTTP_RULE_RES_FYIELD,    /* forced yield, not because of missing data */
 	HTTP_RULE_RES_STOP,      /* stopped processing on an accept */
 	HTTP_RULE_RES_DENY,      /* deny (or tarpit if TX_CLTARPIT)  */
 	HTTP_RULE_RES_ABRT,      /* abort request, msg already sent (eg: auth) */
