@@ -40,30 +40,10 @@ int peers_register_table(struct peers *, struct stktable *table);
 void peers_setup_frontend(struct proxy *fe);
 void peers_register_keywords(struct peers_kw_list *pkwl);
 
-#if defined(USE_OPENSSL)
 static inline enum obj_type *peer_session_target(struct peer *p, struct stream *s)
 {
-	if (p->srv->use_ssl)
-		return &p->srv->obj_type;
-	else
-		return &s->be->obj_type;
+	return &p->srv->obj_type;
 }
 
-static inline struct xprt_ops *peer_xprt(struct peer *p)
-{
-	return p->srv->use_ssl ? xprt_get(XPRT_SSL) : xprt_get(XPRT_RAW);
-}
-#else
-static inline enum obj_type *peer_session_target(struct peer *p, struct stream *s)
-{
-	return &s->be->obj_type;
-}
-
-static inline struct xprt_ops *peer_xprt(struct peer *p)
-{
-	return xprt_get(XPRT_RAW);
-}
-#endif
-
 #endif /* _HAPROXY_PEERS_H */
 
