@@ -224,7 +224,7 @@ static forceinline char *sc_show_flags(char *buf, size_t len, const char *delim,
 	_(SC_FL_NEED_BUFF, _(SC_FL_NEED_ROOM,
         _(SC_FL_RCV_ONCE, _(SC_FL_SND_ASAP, _(SC_FL_SND_NEVERWAIT, _(SC_FL_SND_EXP_MORE,
 	_(SC_FL_ABRT_WANTED, _(SC_FL_SHUT_WANTED, _(SC_FL_ABRT_DONE, _(SC_FL_SHUT_DONE,
-	_(SC_FL_EOS, _(SC_FL_HAVE_BUFF, _(SC_FL_NO_FASTFWD)))))))))))))))))))));
+	_(SC_FL_EOS, _(SC_FL_HAVE_BUFF))))))))))))))))))));
 	/* epilogue */
 	_(~0U);
 	return buf;
@@ -318,10 +318,10 @@ struct sedesc {
 	struct stconn *sc;         /* the stream connector we're attached to, or NULL */
 	struct iobuf iobuf;        /* contains data forwarded by the other side and that must be sent by the stream endpoint */
 	unsigned int flags;        /* SE_FL_* */
+	uint32_t term_evts_log;    /* Termination events log: first 4 events reported */
 	struct se_abort_info abort_info; /* Info about abort, as reported by the endpoint and eventually enriched by the app level */
 	unsigned int lra;          /* the last read activity */
 	unsigned int fsb;          /* the first send blocked */
-	/* 4 bytes hole here */
 	struct xref xref;          /* cross reference with the opposite SC */
 };
 
@@ -348,6 +348,7 @@ struct stconn {
 
 	unsigned int flags;                  /* SC_FL_* */
 	unsigned int ioto;                   /* I/O activity timeout */
+	uint32_t term_evts_log;              /* termination events log aggregating SE + connection events */
 	ssize_t room_needed;                 /* free space in the input buffer required to receive more data.
 					      *    -1   : the SC is waiting for room but not on a specific amount of data
 					      *    >= 0 : min free space required to progress. 0 means SC must be unblocked ASAP
