@@ -11,7 +11,6 @@
 
 #include <import/ebsttree.h>
 #include <import/mjson.h>
-#include <import/mt_list.h>
 
 #include <haproxy/acme-t.h>
 
@@ -140,7 +139,8 @@ static void acme_trace(enum trace_level level, uint64_t mask, const struct trace
 	}
 }
 
-struct mt_list acme_tasks = MT_LIST_HEAD_INIT(acme_tasks);
+struct eb_root acme_tasks = EB_ROOT_UNIQUE;
+__decl_thread(HA_RWLOCK_T acme_lock);
 
 static struct acme_cfg *acme_cfgs = NULL;
 static struct acme_cfg *cur_acme = NULL;
@@ -277,14 +277,6 @@ static int cfg_parse_acme(const char *file, int linenum, char **args, int kwm)
 			goto out;
 		}
 
-		if (httpclient_acme_px == NULL) {
-			if (httpclient_acme_init() & ERR_FATAL) {
-				err_code |= ERR_ALERT | ERR_FATAL;
-				ha_alert("parsing [%s:%d]: out of memory.\n", file, linenum);
-				goto out;
-			}
-		}
-
 		cur_acme = new_acme_cfg(args[1]);
 		if (!cur_acme) {
 			err_code |= ERR_ALERT | ERR_FATAL;
@@ -707,7 +699,7 @@ static int cfg_postsection_acme()
 		}
 
 		if ((bio = BIO_new_file(store->path, "w+")) == NULL) {
-			ha_alert("acme: out of memory.\n");
+			ha_alert("acme: cannot create the file '%s', check your permissions.\n", cur_acme->account.file);
 			err_code |= ERR_ALERT | ERR_FATAL | ERR_ABORT;
 			goto out;
 		}
@@ -749,6 +741,21 @@ out:
 	return err_code;
 }
 
+/* initialize the httpclient just before check_config_validity() because it could create a defaults resolver if it
+ * doesn't exist. */
+static int cfg_precheck_acme()
+{
+	if (acme_cfgs) {
+		if (httpclient_acme_init() & ERR_FATAL) {
+			ha_alert("couldn't initialize the httpclient for ACME.\n");
+			return ERR_ABORT;
+
+		}
+	}
+	return ERR_NONE;
+}
+REGISTER_PRE_CHECK(cfg_precheck_acme);
+
 /* postparser function checks if the ACME section was declared */
 static int cfg_postparser_acme()
 {
@@ -765,6 +772,16 @@ static int cfg_postparser_acme()
 			ha_alert("acme '%s' was used on a crt line [%s:%d], but no '%s' section exists!\n",
 			         tmp_acme->name, tmp_acme->filename, -tmp_acme->linenum, tmp_acme->name);
 		}
+		if (tmp_acme->map) {
+			struct pat_ref *ref;
+
+			ref = pat_ref_lookup(tmp_acme->map);
+			if (!ref) {
+				ret++;
+				ha_alert("acme section '%s' line [%s:%d] has the map '%s' configured, but this map doesn't exist\n",
+				         tmp_acme->name, tmp_acme->filename, tmp_acme->linenum, tmp_acme->map);
+			}
+		}
 		tmp_acme = tmp_acme->next;
 	}
 
@@ -1699,7 +1716,7 @@ int acme_res_auth(struct task *task, struct acme_ctx *ctx, struct acme_auth *aut
 			}
 
 			send_log(NULL, LOG_NOTICE,"acme: %s: dns-01 requires to set the \"_acme-challenge.%.*s\" TXT record to \"%.*s\" and use the \"acme challenge_ready %s domain %.*s\" command over the CLI\n",
-			                                             ctx->store->path, (int)auth->dns.len, auth->dns.ptr, (int)trash.data, trash.area, ctx->store->path, (int)auth->dns.len, auth->dns.ptr);
+			                                             ctx->store->path, (int)auth->dns.len, auth->dns.ptr, (int)dns_record->data, dns_record->area, ctx->store->path, (int)auth->dns.len, auth->dns.ptr);
 
 			/* dump to the "dpapi" sink */
 			line[nmsg++] = ist("acme deploy ");
@@ -2148,7 +2165,6 @@ struct task *acme_process(struct task *task, void *context, unsigned int state)
 	enum acme_st st = ctx->state;
 	enum http_st http_st = ctx->http_state;
 	char *errmsg = NULL;
-	struct mt_list tmp = MT_LIST_LOCK_FULL(&ctx->el);
 
 re:
 	TRACE_USER("ACME Task Handle", ACME_EV_TASK, ctx, &st);
@@ -2340,7 +2356,6 @@ re:
 	}
 
 	/* this is called after initializing a request */
-	MT_LIST_UNLOCK_FULL(&ctx->el, tmp);
 	ctx->http_state = http_st;
 	ctx->state = st;
 	task->expire = TICK_ETERNITY;
@@ -2360,7 +2375,6 @@ nextreq:
 	task->expire = tick_add(now_ms, ctx->retryafter * 1000);
 	ctx->retryafter = 0;
 
-	MT_LIST_UNLOCK_FULL(&ctx->el, tmp);
 	return task;
 
 retry:
@@ -2391,7 +2405,6 @@ retry:
 
 	ha_free(&errmsg);
 
-	MT_LIST_UNLOCK_FULL(&ctx->el, tmp);
 	return task;
 
 abort:
@@ -2400,9 +2413,9 @@ abort:
 
 end:
 	acme_del_acme_ctx_map(ctx);
-	/* unlink ctx from the mtlist then destroy */
-	mt_list_unlock_link(tmp);
-	mt_list_unlock_self(&ctx->el);
+	HA_RWLOCK_WRLOCK(OTHER_LOCK, &acme_lock);
+	ebmb_delete(&ctx->node);
+	HA_RWLOCK_WRUNLOCK(OTHER_LOCK, &acme_lock);
 	acme_ctx_destroy(ctx);
 	task_destroy(task);
 	task = NULL;
@@ -2414,8 +2427,6 @@ wait:
 	ctx->http_state = ACME_HTTP_REQ;
 	ctx->state = st;
 	task->expire = TICK_ETERNITY;
-
-	MT_LIST_UNLOCK_FULL(&ctx->el, tmp);
 	return task;
 }
 
@@ -2619,11 +2630,6 @@ static int acme_start_task(struct ckch_store *store, char **errmsg)
 	struct ckch_store *newstore = NULL;
 	EVP_PKEY *pkey = NULL;
 
-	if (store->acme_task != NULL) {
-		memprintf(errmsg, "An ACME task is already running for certificate '%s'.", store->path);
-		goto err;
-	}
-
 	if (!store->conf.acme.domains) {
 		memprintf(errmsg, "No 'domains' were configured for certificate. ");
 		goto err;
@@ -2647,18 +2653,23 @@ static int acme_start_task(struct ckch_store *store, char **errmsg)
 	task->nice = 0;
 	task->process = acme_process;
 
-        /* register the task in the store so we don't
-	 * have 2 tasks at the same time
-	 */
-        store->acme_task = task;
-
 	/* XXX: following init part could be done in the task */
-	ctx = calloc(1, sizeof *ctx);
+	ctx = calloc(1, sizeof *ctx + strlen(newstore->path) + 1);
 	if (!ctx) {
 		memprintf(errmsg, "Out of memory.");
 		goto err;
 	}
 
+	memcpy(ctx->name, newstore->path, strlen(newstore->path) + 1);
+
+	HA_RWLOCK_WRLOCK(OTHER_LOCK, &acme_lock);
+	if (ebst_insert(&acme_tasks, &ctx->node) != &ctx->node) {
+		memprintf(errmsg, "%sTask already exists for '%s' certificate.\n", *errmsg ? *errmsg : "", newstore->path);
+		HA_RWLOCK_WRUNLOCK(OTHER_LOCK, &acme_lock);
+		goto err;
+	}
+	HA_RWLOCK_WRUNLOCK(OTHER_LOCK, &acme_lock);
+
 	/* set the number of remaining retries when facing an error */
 	ctx->retries = ACME_RETRY;
 
@@ -2682,9 +2693,6 @@ static int acme_start_task(struct ckch_store *store, char **errmsg)
 	task->context = ctx;
 	ctx->task = task;
 
-	MT_LIST_INIT(&ctx->el);
-	MT_LIST_APPEND(&acme_tasks, &ctx->el);
-
 	send_log(NULL, LOG_NOTICE, "acme: %s: Starting update of the certificate.\n", ctx->store->path);
 
 	TRACE_USER("ACME Task start", ACME_EV_NEW, ctx);
@@ -2695,7 +2703,12 @@ static int acme_start_task(struct ckch_store *store, char **errmsg)
 err:
 	EVP_PKEY_free(pkey);
 	ckch_store_free(newstore);
-	acme_ctx_destroy(ctx);
+	if (ctx) {
+		HA_RWLOCK_WRLOCK(OTHER_LOCK, &acme_lock);
+		ebmb_delete(&ctx->node);
+		HA_RWLOCK_WRUNLOCK(OTHER_LOCK, &acme_lock);
+		acme_ctx_destroy(ctx);
+	}
 	memprintf(errmsg, "%sCan't start the ACME client.", *errmsg ? *errmsg : "");
 	return 1;
 }
@@ -2738,10 +2751,10 @@ static int cli_acme_chall_ready_parse(char **args, char *payload, struct appctx
 	char *errmsg = NULL;
 	const char *crt;
 	const char *dns;
-	struct mt_list back;
 	struct acme_ctx *ctx;
 	struct acme_auth *auth;
 	int found = 0;
+	struct ebmb_node *node = NULL;
 
 	if (!*args[2] && !*args[3] && !*args[4]) {
 		memprintf(&errmsg, ": not enough parameters\n");
@@ -2751,8 +2764,10 @@ static int cli_acme_chall_ready_parse(char **args, char *payload, struct appctx
 	crt = args[2];
 	dns = args[4];
 
-
-	MT_LIST_FOR_EACH_ENTRY_LOCKED(ctx, &acme_tasks, el, back) {
+	HA_RWLOCK_RDLOCK(OTHER_LOCK, &acme_lock);
+	node = ebmb_first(&acme_tasks);
+	while (node) {
+		ctx = ebmb_entry(node, struct acme_ctx, node);
 
 		if (strcmp(ctx->store->path, crt) != 0)
 			continue;
@@ -2771,7 +2786,9 @@ static int cli_acme_chall_ready_parse(char **args, char *payload, struct appctx
 			}
 			auth = auth->next;
 		}
+		node = ebmb_next(node);
 	}
+	HA_RWLOCK_RDUNLOCK(OTHER_LOCK, &acme_lock);
 	if (!found) {
 		memprintf(&errmsg, "Couldn't find the ACME task using crt \"%s\" and dns \"%s\" !\n", crt, dns);
 		goto err;
@@ -2811,7 +2828,11 @@ static int cli_acme_status_io_handler(struct appctx *appctx)
 			time_t notAfter = 0;
 			time_t sched = 0;
 			ullong remain = 0;
-			int running = !!store->acme_task;
+			int running = 0;
+
+			HA_RWLOCK_RDLOCK(OTHER_LOCK, &acme_lock);
+			running = !!ebst_lookup(&acme_tasks, store->path);
+			HA_RWLOCK_RDUNLOCK(OTHER_LOCK, &acme_lock);
 
 			if (global_ssl.acme_scheduler)
 				state = "Scheduled";
@@ -2873,6 +2894,7 @@ INITCALL1(STG_REGISTER, cli_register_kw, &cli_kws);
 static void __acme_init(void)
 {
 	hap_register_feature("ACME");
+	HA_RWLOCK_INIT(&acme_lock);
 }
 INITCALL0(STG_REGISTER, __acme_init);
 
