@@ -541,8 +541,18 @@ size_t appctx_rcv_buf(struct stconn *sc, struct buffer *buf, size_t count, unsig
 		applet_fl_clr(appctx, APPCTX_FL_OUTBLK_FULL);
 
 	if (b_data(&appctx->outbuf)) {
-		se_fl_set(appctx->sedesc, SE_FL_RCV_MORE | SE_FL_WANT_ROOM);
-		TRACE_STATE("waiting for more room", APPLET_EV_RECV|APPLET_EV_BLK, appctx);
+		se_fl_set(appctx->sedesc, SE_FL_RCV_MORE);
+		if (b_data(buf)) {
+			/* some data left with other data in the buffer often
+			 * means the applet couldn't write a block at once.
+			 */
+			se_fl_set(appctx->sedesc, SE_FL_WANT_ROOM);
+			TRACE_STATE("waiting for more room", APPLET_EV_RECV|APPLET_EV_BLK, appctx);
+		} else {
+			/* the applet just doesn't want to give us partial data */
+			se_fl_clr(appctx->sedesc, SE_FL_WANT_ROOM);
+			TRACE_STATE("waiting for more data from applet", APPLET_EV_RECV|APPLET_EV_BLK, appctx);
+		}
 	}
 	else {
 		se_fl_clr(appctx->sedesc, SE_FL_RCV_MORE | SE_FL_WANT_ROOM);
@@ -952,7 +962,7 @@ struct task *task_process_applet(struct task *t, void *context, unsigned int sta
 	}
 
 	/* measure the call rate and check for anomalies when too high */
-	if (!did_recv && !did_send) {
+	if ((!did_recv && sc_is_send_allowed(sc)) && (!did_send && sc_is_recv_allowed(sc))) {
 		rate = update_freq_ctr(&app->call_rate, 1);
 		if (rate >= 100000 && app->call_rate.prev_ctr) // looped like this more than 100k times over last second
 			stream_dump_and_crash(&app->obj_type, read_freq_ctr(&app->call_rate));
