@@ -14,7 +14,6 @@
  * libcap. Let's stick to what we need and the kernel documents (capset).
  * Note that prctl is needed here.
  */
-#include <linux/capability.h>
 #include <sys/prctl.h>
 #include <errno.h>
 #include <unistd.h>
@@ -24,8 +23,14 @@
 #include <haproxy/cfgparse.h>
 #include <haproxy/errors.h>
 #include <haproxy/global.h>
+#include <haproxy/linuxcap.h>
 #include <haproxy/tools.h>
 
+struct __user_cap_header_struct cap_hdr_haproxy = {
+	.pid = 0, /* current process */
+	.version = _LINUX_CAPABILITY_VERSION_3,
+};
+
 /* supported names, zero-terminated */
 static const struct {
 	int cap;
@@ -47,12 +52,6 @@ static const struct {
 	{ 0, 0 }
 };
 
-/* provided by sys/capability.h on some distros */
-static inline int capget(cap_user_header_t hdrp, const cap_user_data_t datap)
-{
-	return syscall(SYS_capget, hdrp, datap);
-}
-
 /* provided by sys/capability.h on some distros */
 static inline int capset(cap_user_header_t hdrp, const cap_user_data_t datap)
 {
@@ -81,13 +80,14 @@ static uint32_t caplist;
  * LSTCHK_NETADM/LSTCHK_SYSADM is presented in global.last_checks and some
  * failures are encountered.
  */
-int prepare_caps_from_permitted_set(int from_uid, int to_uid, const char *program_name)
+int prepare_caps_from_permitted_set(int from_uid, int to_uid)
 {
-	struct __user_cap_data_struct start_cap_data = { };
-	struct __user_cap_header_struct cap_hdr = {
-		.pid = 0, /* current process */
-		.version = _LINUX_CAPABILITY_VERSION_1,
-	};
+	/* _LINUX_CAPABILITY_U32S_1 = 1 and corresponds to version 1, which is three
+	 * 32-bit integers set. So kernel in capset()/capget() will copy_from/to_user
+	 * only _LINUX_CAPABILITY_U32S_1 * (sizeof(struct __user_cap_data_struct)),
+	 * i.e. only the __user_cap_data_struct[0].
+	 */
+	struct __user_cap_data_struct start_cap_data[_LINUX_CAPABILITY_U32S_3] = { };
 
 	/* started as root */
 	if (!from_uid)
@@ -102,20 +102,20 @@ int prepare_caps_from_permitted_set(int from_uid, int to_uid, const char *progra
 	 * these capabilities and the file effective bit on haproxy binary via
 	 * setcap, see capabilities man page for details.
 	 */
-	if (capget(&cap_hdr, &start_cap_data) == -1) {
+	if (capget(&cap_hdr_haproxy, start_cap_data) == -1) {
 		if (global.last_checks & (LSTCHK_NETADM | LSTCHK_SYSADM))
 			ha_diag_warning("Failed to get process capabilities using capget(): %s. "
 					"Can't use capabilities that might be set on %s binary "
-					"by administrator.\n", strerror(errno), program_name);
+					"by administrator.\n", strerror(errno), progname);
 		return 0;
 	}
 
-	if (start_cap_data.effective & ((1 << CAP_NET_ADMIN)|(1 << CAP_NET_RAW))) {
+	if (start_cap_data[0].effective & ((1 << CAP_NET_ADMIN)|(1 << CAP_NET_RAW))) {
 		global.last_checks &= ~LSTCHK_NETADM;
 		return 0;
 	}
 
-	if (start_cap_data.effective & ((1 << CAP_SYS_ADMIN))) {
+	if (start_cap_data[0].effective & ((1 << CAP_SYS_ADMIN))) {
 		global.last_checks &= ~LSTCHK_SYSADM;
 		return 0;
 	}
@@ -125,9 +125,9 @@ int prepare_caps_from_permitted_set(int from_uid, int to_uid, const char *progra
 	 * set, if it is in the caplist and also presented in the binary
 	 * permitted set.
 	 */
-	if (caplist && start_cap_data.permitted & caplist) {
-		start_cap_data.effective |= start_cap_data.permitted & caplist;
-		if (capset(&cap_hdr, &start_cap_data) == 0) {
+	if (caplist && start_cap_data[0].permitted & caplist) {
+		start_cap_data[0].effective |= start_cap_data[0].permitted & caplist;
+		if (capset(&cap_hdr_haproxy, start_cap_data) == 0) {
 			if (caplist & ((1 << CAP_NET_ADMIN)|(1 << CAP_NET_RAW)))
 				global.last_checks &= ~LSTCHK_NETADM;
 			if (caplist & (1 << CAP_SYS_ADMIN))
@@ -135,7 +135,7 @@ int prepare_caps_from_permitted_set(int from_uid, int to_uid, const char *progra
 		} else if (global.last_checks & (LSTCHK_NETADM|LSTCHK_SYSADM)) {
 			ha_diag_warning("Failed to put capabilities from caplist in %s "
 					"process Effective capabilities set using capset(): %s\n",
-					program_name, strerror(errno));
+					progname, strerror(errno));
 		}
 	}
 
@@ -157,11 +157,12 @@ int prepare_caps_from_permitted_set(int from_uid, int to_uid, const char *progra
  */
 int prepare_caps_for_setuid(int from_uid, int to_uid)
 {
-	struct __user_cap_data_struct cap_data = { };
-	struct __user_cap_header_struct cap_hdr = {
-		.pid = 0, /* current process */
-		.version = _LINUX_CAPABILITY_VERSION_1,
-	};
+	/* _LINUX_CAPABILITY_U32S_1 = 1 and corresponds to version 1, which is three
+	 * 32-bit integers set. So kernel in capset()/capget() will copy_from/to_user
+	 * only _LINUX_CAPABILITY_U32S_1 * (sizeof(struct __user_cap_data_struct)),
+	 * i.e. only the __user_cap_data_struct[0].
+	 */
+	struct __user_cap_data_struct cap_data[_LINUX_CAPABILITY_U32S_3] = { };
 
 	if (from_uid != 0)
 		return 0;
@@ -177,8 +178,8 @@ int prepare_caps_for_setuid(int from_uid, int to_uid)
 		return -1;
 	}
 
-	cap_data.effective = cap_data.permitted = caplist | (1 << CAP_SETUID);
-	if (capset(&cap_hdr, &cap_data) == -1) {
+	cap_data[0].effective = cap_data[0].permitted = caplist | (1 << CAP_SETUID);
+	if (capset(&cap_hdr_haproxy, cap_data) == -1) {
 		ha_alert("Failed to preset the capabilities to preserve using capset(): %s\n", strerror(errno));
 		return -1;
 	}
@@ -188,8 +189,8 @@ int prepare_caps_for_setuid(int from_uid, int to_uid)
 		return -1;
 	}
 
-	cap_data.effective = cap_data.permitted = caplist | (1 << CAP_SETUID);
-	if (capset(&cap_hdr, &cap_data) == -1) {
+	cap_data[0].effective = cap_data[0].permitted = caplist | (1 << CAP_SETUID);
+	if (capset(&cap_hdr_haproxy, cap_data) == -1) {
 		ha_alert("Failed to set the final capabilities using capset(): %s\n", strerror(errno));
 		return -1;
 	}
@@ -210,11 +211,7 @@ int prepare_caps_for_setuid(int from_uid, int to_uid)
  */
 int finalize_caps_after_setuid(int from_uid, int to_uid)
 {
-	struct __user_cap_data_struct cap_data = { };
-	struct __user_cap_header_struct cap_hdr = {
-		.pid = 0, /* current process */
-		.version = _LINUX_CAPABILITY_VERSION_1,
-	};
+	struct __user_cap_data_struct cap_data[_LINUX_CAPABILITY_U32S_3] = { };
 
 	if (from_uid != 0)
 		return 0;
@@ -225,11 +222,12 @@ int finalize_caps_after_setuid(int from_uid, int to_uid)
 	if (!caplist)
 		return 0;
 
-	cap_data.effective = cap_data.permitted = caplist;
-	if (capset(&cap_hdr, &cap_data) == -1) {
+	cap_data[0].effective = cap_data[0].permitted = caplist;
+	if (capset(&cap_hdr_haproxy, cap_data) == -1) {
 		ha_alert("Failed to drop the setuid capability using capset(): %s\n", strerror(errno));
 		return -1;
 	}
+
 	/* all's good */
 	return 0;
 }
