@@ -50,8 +50,9 @@ struct connection *sockpair_accept_conn(struct listener *l, int *status);
 
 struct proto_fam proto_fam_sockpair = {
 	.name = "sockpair",
-	.sock_domain = AF_CUST_SOCKPAIR,
-	.sock_family = AF_UNIX,
+	.sock_domain = AF_UNIX,
+	.sock_family = AF_CUST_SOCKPAIR,
+	.real_family = AF_CUST_SOCKPAIR,
 	.sock_addrlen = sizeof(struct sockaddr_un),
 	.l3_addrlen = sizeof(((struct sockaddr_un*)0)->sun_path),
 	.addrcmp = NULL,
@@ -94,8 +95,6 @@ struct protocol proto_sockpair = {
 	.rx_unbind      = sock_unbind,
 	.rx_listening   = sockpair_accepting_conn,
 	.default_iocb   = sock_accept_iocb,
-	.receivers      = LIST_HEAD_INIT(proto_sockpair.receivers),
-	.nb_receivers   = 0,
 };
 
 INITCALL1(STG_REGISTER, protocol_register, &proto_sockpair);
@@ -152,6 +151,12 @@ int sockpair_bind_receiver(struct receiver *rx, char **errmsg)
 			err |= ERR_RETRYABLE;
 			goto bind_ret_err;
 		}
+		/* taking the other one's FD will result in it being marked
+		 * extern and being dup()ed. Let's mark the receiver as
+		 * inherited so that it properly bypasses all second-stage
+		 * setup and avoids being passed to new processes.
+		 */
+		rx->flags |= RX_F_INHERITED;
 		rx->fd = rx->shard_info->ref->fd;
 	}
 
@@ -234,12 +239,12 @@ static int sockpair_bind_listener(struct listener *listener, char *errmsg, int e
  */
 int send_fd_uxst(int fd, int send_fd)
 {
-	char iobuf[2];
+	char iobuf[2] = {0};
 	struct iovec iov;
 	struct msghdr msghdr;
 
-	char cmsgbuf[CMSG_SPACE(sizeof(int))];
-	char buf[CMSG_SPACE(sizeof(int))];
+	char cmsgbuf[CMSG_SPACE(sizeof(int))] = {0};
+	char buf[CMSG_SPACE(sizeof(int))] = {0};
 	struct cmsghdr *cmsg = (void *)buf;
 
 	int *fdptr;
