@@ -46,8 +46,9 @@ static void quic_dflt_transport_params_cpy(struct quic_transport_params *dst)
  */
 void quic_transport_params_init(struct quic_transport_params *p, int server)
 {
-	const uint64_t ncb_size = global.tune.bufsize - NCB_RESERVED_SZ;
+	const uint64_t stream_rx_bufsz = qmux_stream_rx_bufsz();
 	const int max_streams_bidi = global.tune.quic_frontend_max_streams_bidi;
+	/* TODO value used to conform with HTTP/3, should be derived from app_ops */
 	const int max_streams_uni = 3;
 
 	/* Set RFC default values for unspecified parameters. */
@@ -62,12 +63,33 @@ void quic_transport_params_init(struct quic_transport_params *p, int server)
 	else
 		p->max_idle_timeout = global.tune.quic_backend_max_idle_timeout;
 
-	p->initial_max_streams_bidi            = max_streams_bidi;
-	p->initial_max_streams_uni             = max_streams_uni;
-	p->initial_max_stream_data_bidi_local  = ncb_size;
-	p->initial_max_stream_data_bidi_remote = ncb_size;
-	p->initial_max_stream_data_uni         = ncb_size;
-	p->initial_max_data = (max_streams_bidi + max_streams_uni) * ncb_size;
+	/* Set limit on number of concurrently opened streams. */
+	p->initial_max_streams_bidi = max_streams_bidi;
+	p->initial_max_streams_uni  = max_streams_uni;
+
+	/* Set connection flow-control data limit, either from configuration,
+	 * or automatically calculated from max number of concurrently opened
+	 * streams.
+	 */
+	if (global.tune.quic_frontend_max_data)
+		p->initial_max_data = global.tune.quic_frontend_max_data;
+	else
+		p->initial_max_data = max_streams_bidi * stream_rx_bufsz;
+
+	/* Set remote streams flow-control data limit. This is calculated as a
+	 * ratio from max-data, then rounded up to bufsize.
+	 */
+	p->initial_max_stream_data_bidi_remote =
+	  p->initial_max_data * global.tune.quic_frontend_stream_data_ratio / 100;
+	p->initial_max_stream_data_bidi_remote =
+	  stream_rx_bufsz * ((p->initial_max_stream_data_bidi_remote + (stream_rx_bufsz - 1)) / stream_rx_bufsz);
+
+	/* Set remaining flow-control data limit. Local bidi streams are unused
+	 * on server side. Uni streams are only used for control exchange, so
+	 * only a single buffer for in flight data should be enough.
+	 */
+	p->initial_max_stream_data_bidi_local  = stream_rx_bufsz;
+	p->initial_max_stream_data_uni         = stream_rx_bufsz;
 
 	if (server) {
 		p->with_stateless_reset_token  = 1;
@@ -224,17 +246,27 @@ static int quic_transport_param_dec_version_info(struct tp_version_information *
  * must be set to 1 for a server (haproxy listener) or 0 for a client (connection
  * to an haproxy server).
  */
-static int quic_transport_param_decode(struct quic_transport_params *p,
-                                       int server, uint64_t type,
-                                       const unsigned char **buf, size_t len)
+static enum quic_tp_dec_err
+quic_transport_param_decode(struct quic_transport_params *p, int server,
+                            uint64_t type, const unsigned char **buf, size_t len)
 {
 	const unsigned char *end = *buf + len;
 
 	switch (type) {
 	case QUIC_TP_ORIGINAL_DESTINATION_CONNECTION_ID:
-		if (!server || len > sizeof p->original_destination_connection_id.data)
-			return 0;
+		/* RFC 9000 18.2. Transport Parameter Definitions
+		 *
+		 * A client MUST NOT include any server-only transport parameter:
+		 * original_destination_connection_id, preferred_address,
+		 * retry_source_connection_id, or stateless_reset_token. A server MUST
+		 * treat receipt of any of these transport parameters as a connection
+		 * error of type TRANSPORT_PARAMETER_ERROR.
+		 */
+		if (!server)
+			return QUIC_TP_DEC_ERR_INVAL;
 
+		if (len > sizeof p->original_destination_connection_id.data)
+			return QUIC_TP_DEC_ERR_TRUNC;
 		if (len)
 			memcpy(p->original_destination_connection_id.data, *buf, len);
 		p->original_destination_connection_id.len = len;
@@ -243,7 +275,7 @@ static int quic_transport_param_decode(struct quic_transport_params *p,
 		break;
 	case QUIC_TP_INITIAL_SOURCE_CONNECTION_ID:
 		if (len > sizeof p->initial_source_connection_id.data)
-			return 0;
+			return QUIC_TP_DEC_ERR_TRUNC;
 
 		if (len)
 			memcpy(p->initial_source_connection_id.data, *buf, len);
@@ -252,81 +284,120 @@ static int quic_transport_param_decode(struct quic_transport_params *p,
 		p->initial_source_connection_id_present = 1;
 		break;
 	case QUIC_TP_STATELESS_RESET_TOKEN:
-		if (!server || len != sizeof p->stateless_reset_token)
-			return 0;
+		/* see original_destination_connection_id RFC reference above. */
+		if (!server)
+			return QUIC_TP_DEC_ERR_INVAL;
+
+		if (len != sizeof p->stateless_reset_token)
+			return QUIC_TP_DEC_ERR_TRUNC;
 		memcpy(p->stateless_reset_token, *buf, len);
 		*buf += len;
 		p->with_stateless_reset_token = 1;
 		break;
 	case QUIC_TP_PREFERRED_ADDRESS:
+		/* see original_destination_connection_id RFC reference above. */
 		if (!server)
-			return 0;
+			return QUIC_TP_DEC_ERR_INVAL;
+
 		if (!quic_transport_param_dec_pref_addr(&p->preferred_address, buf, *buf + len))
-			return 0;
+			return QUIC_TP_DEC_ERR_TRUNC;
 		p->with_preferred_address = 1;
 		break;
 	case QUIC_TP_MAX_IDLE_TIMEOUT:
 		if (!quic_dec_int(&p->max_idle_timeout, buf, end))
-			return 0;
+			return QUIC_TP_DEC_ERR_TRUNC;
 		break;
 	case QUIC_TP_MAX_UDP_PAYLOAD_SIZE:
 		if (!quic_dec_int(&p->max_udp_payload_size, buf, end))
-			return 0;
+			return QUIC_TP_DEC_ERR_TRUNC;
+
+		/* RFC 9000 18.2. Transport Parameter Definitions
+		 *
+		 * max_udp_payload_size (0x03): [...]
+		 * The default for this parameter is the maximum permitted UDP
+		 * payload of 65527. Values below 1200 are invalid.
+		 */
+		if (p->max_udp_payload_size < 1200)
+			return QUIC_TP_DEC_ERR_INVAL;
+
 		break;
 	case QUIC_TP_INITIAL_MAX_DATA:
 		if (!quic_dec_int(&p->initial_max_data, buf, end))
-			return 0;
+			return QUIC_TP_DEC_ERR_TRUNC;
 		break;
 	case QUIC_TP_INITIAL_MAX_STREAM_DATA_BIDI_LOCAL:
 		if (!quic_dec_int(&p->initial_max_stream_data_bidi_local, buf, end))
-			return 0;
+			return QUIC_TP_DEC_ERR_TRUNC;
 		break;
 	case QUIC_TP_INITIAL_MAX_STREAM_DATA_BIDI_REMOTE:
 		if (!quic_dec_int(&p->initial_max_stream_data_bidi_remote, buf, end))
-			return 0;
+			return QUIC_TP_DEC_ERR_TRUNC;
 		break;
 	case QUIC_TP_INITIAL_MAX_STREAM_DATA_UNI:
 		if (!quic_dec_int(&p->initial_max_stream_data_uni, buf, end))
-			return 0;
+			return QUIC_TP_DEC_ERR_TRUNC;
 		break;
 	case QUIC_TP_INITIAL_MAX_STREAMS_BIDI:
 		if (!quic_dec_int(&p->initial_max_streams_bidi, buf, end))
-			return 0;
+			return QUIC_TP_DEC_ERR_TRUNC;
 		break;
 	case QUIC_TP_INITIAL_MAX_STREAMS_UNI:
 		if (!quic_dec_int(&p->initial_max_streams_uni, buf, end))
-			return 0;
+			return QUIC_TP_DEC_ERR_TRUNC;
 		break;
 	case QUIC_TP_ACK_DELAY_EXPONENT:
-		if (!quic_dec_int(&p->ack_delay_exponent, buf, end) ||
-			p->ack_delay_exponent > QUIC_TP_ACK_DELAY_EXPONENT_LIMIT)
-			return 0;
+		if (!quic_dec_int(&p->ack_delay_exponent, buf, end))
+			return QUIC_TP_DEC_ERR_TRUNC;
+
+		/* RFC 9000 18.2. Transport Parameter Definitions
+		 *
+		 * ack_delay_exponent (0x0a): [...]
+		 * Values above 20 are invalid.
+		 */
+		if (p->ack_delay_exponent > QUIC_TP_ACK_DELAY_EXPONENT_LIMIT)
+			return QUIC_TP_DEC_ERR_INVAL;
+
 		break;
 	case QUIC_TP_MAX_ACK_DELAY:
-		if (!quic_dec_int(&p->max_ack_delay, buf, end) ||
-			p->max_ack_delay > QUIC_TP_MAX_ACK_DELAY_LIMIT)
-			return 0;
+		if (!quic_dec_int(&p->max_ack_delay, buf, end))
+			return QUIC_TP_DEC_ERR_TRUNC;
+
+		/* RFC 9000 18.2. Transport Parameter Definitions
+		 *
+		 * max_ack_delay (0x0b): [...]
+		 * Values of 2^14 or greater are invalid.
+		 */
+		if (p->max_ack_delay >= QUIC_TP_MAX_ACK_DELAY_LIMIT)
+			return QUIC_TP_DEC_ERR_INVAL;
+
 		break;
 	case QUIC_TP_DISABLE_ACTIVE_MIGRATION:
 		/* Zero-length parameter type. */
 		if (len != 0)
-			return 0;
+			return QUIC_TP_DEC_ERR_TRUNC;
 		p->disable_active_migration = 1;
 		break;
 	case QUIC_TP_ACTIVE_CONNECTION_ID_LIMIT:
 		if (!quic_dec_int(&p->active_connection_id_limit, buf, end))
-			return 0;
+			return QUIC_TP_DEC_ERR_TRUNC;
 		break;
 	case QUIC_TP_VERSION_INFORMATION:
 		if (!quic_transport_param_dec_version_info(&p->version_information,
 		                                           buf, *buf + len, server))
-			return 0;
+			return QUIC_TP_DEC_ERR_TRUNC;
+		break;
+	case QUIC_TP_RETRY_SOURCE_CONNECTION_ID:
+		/* see original_destination_connection_id RFC reference above. */
+		if (!server)
+			return QUIC_TP_DEC_ERR_INVAL;
+
+		/* TODO implement parsing for client side */
 		break;
 	default:
 		*buf += len;
 	};
 
-	return *buf == end;
+	return *buf == end ? QUIC_TP_DEC_ERR_NONE : QUIC_TP_DEC_ERR_TRUNC;
 }
 
 /* Encode <type> and <len> variable length values in <buf>.
@@ -576,10 +647,11 @@ int quic_transport_params_encode(unsigned char *buf,
  * or 0 for a client (connection to a haproxy server).
  * Returns 1 if succeeded, 0 if not.
  */
-static int quic_transport_params_decode(struct quic_transport_params *p, int server,
-                                        const unsigned char *buf,
-                                        const unsigned char *end)
+static enum quic_tp_dec_err
+quic_transport_params_decode(struct quic_transport_params *p, int server,
+                             const unsigned char *buf, const unsigned char *end)
 {
+	enum quic_tp_dec_err err;
 	const unsigned char *pos;
 	uint64_t type, len = 0;
 
@@ -587,31 +659,41 @@ static int quic_transport_params_decode(struct quic_transport_params *p, int ser
 
 	while (pos != end) {
 		if (!quic_transport_param_decode_type_len(&type, &len, &pos, end))
-			return 0;
+			return QUIC_TP_DEC_ERR_TRUNC;
 
 		if (end - pos < len)
-			return 0;
+			return QUIC_TP_DEC_ERR_TRUNC;
 
-		if (!quic_transport_param_decode(p, server, type, &pos, len))
-			return 0;
+		err = quic_transport_param_decode(p, server, type, &pos, len);
+		if (err != QUIC_TP_DEC_ERR_NONE)
+			return err;
 	}
 
-	/*
-	 * A server MUST send original_destination_connection_id transport parameter.
-	 * initial_source_connection_id must be present both for server and client.
+	/* RFC 9000 7.3. Authenticating Connection IDs
+	 *
+	 * An endpoint MUST treat the absence of the
+	 * initial_source_connection_id transport parameter from either endpoint
+	 * or the absence of the original_destination_connection_id transport
+	 * parameter from the server as a connection error of type
+	 * TRANSPORT_PARAMETER_ERROR.
 	 */
-	if ((server && !p->original_destination_connection_id_present) ||
-	    !p->initial_source_connection_id_present)
-		return 0;
+	if (!p->initial_source_connection_id_present ||
+	    (server && !p->original_destination_connection_id_present)) {
+		return QUIC_TP_DEC_ERR_INVAL;
+	}
 
-	/* Note that if not received by the peer, active_connection_id_limit will
-	 * have QUIC_TP_DFLT_ACTIVE_CONNECTION_ID_LIMIT as default value. This
-	 * is also the minimum value for this transport parameter.
+	/* RFC 9000 18.2. Transport Parameter Definitions
+	 *
+	 * active_connection_id_limit (0x0e):
+	 * [...] The value of the
+	 * active_connection_id_limit parameter MUST be at least 2. An
+	 * endpoint that receives a value less than 2 MUST close the
+	 * connection with an error of type TRANSPORT_PARAMETER_ERROR.
 	 */
 	if (p->active_connection_id_limit < QUIC_TP_DFLT_ACTIVE_CONNECTION_ID_LIMIT)
-		return 0;
+		return QUIC_TP_DEC_ERR_INVAL;
 
-	return 1;
+	return QUIC_TP_DEC_ERR_NONE;
 }
 
 /* Store transport parameters found in <buf> buffer into <qc> QUIC connection
@@ -620,12 +702,16 @@ static int quic_transport_params_decode(struct quic_transport_params *p, int ser
  * Note that peer transport parameters are stored in the TX part of the connection:
  * they are used to send packets to the peer with its transport parameters as
  * limitations.
- * Returns 1 if succeeded, 0 if not.
+ *
+ * Returns 1 on success, or 0 if parsing is interrupted on a truncated field.
+ * Note that if invalid values are used, success is returned by this function
+ * but the connection is scheduled for CONNECTION_CLOSE emission.
  */
 int quic_transport_params_store(struct quic_conn *qc, int server,
                                 const unsigned char *buf,
                                 const unsigned char *end)
 {
+	enum quic_tp_dec_err err;
 	struct quic_transport_params *tx_params = &qc->tx.params;
 	struct quic_transport_params *rx_params = &qc->rx.params;
 	/* Initial source connection ID */
@@ -634,8 +720,16 @@ int quic_transport_params_store(struct quic_conn *qc, int server,
 	/* initialize peer TPs to RFC default value */
 	quic_dflt_transport_params_cpy(tx_params);
 
-	if (!quic_transport_params_decode(tx_params, server, buf, end))
+	err = quic_transport_params_decode(tx_params, server, buf, end);
+	if (err == QUIC_TP_DEC_ERR_INVAL) {
+		TRACE_ERROR("invalid transport parameter value", QUIC_EV_TRANSP_PARAMS, qc);
+		quic_set_connection_close(qc, quic_err_transport(QC_ERR_TRANSPORT_PARAMETER_ERROR));
+		return 1;
+	}
+	else if (err == QUIC_TP_DEC_ERR_TRUNC) {
+		TRACE_ERROR("error on transport parameters decoding", QUIC_EV_TRANSP_PARAMS, qc);
 		return 0;
+	}
 
 	/* Update the connection from transport parameters received */
 	if (tx_params->version_information.negotiated_version &&
