@@ -79,6 +79,10 @@ int sock_inet6_v6only_default = 0;
 int sock_inet_tcp_maxseg_default = -1;
 int sock_inet6_tcp_maxseg_default = -1;
 
+/* indicate whether v6 looks reachable (this is only a hint) */
+int sock_inet6_seems_reachable = 0;
+uint last_inet6_check = TICK_ETERNITY;
+
 /* Default MPTCPv4/MPTCPv6 MSS settings. -1=unknown. */
 #ifdef HA_HAVE_MPTCP
 int sock_inet_mptcp_maxseg_default = -1;
@@ -147,9 +151,6 @@ void sock_inet_set_port(struct sockaddr_storage *addr, int port)
  */
 int sock_inet_get_dst(int fd, struct sockaddr *sa, socklen_t salen, int dir)
 {
-	int val __maybe_unused;
-	socklen_t len __maybe_unused;
-
 	if (dir)
 		return getpeername(fd, sa, &salen);
 	else {
@@ -160,19 +161,10 @@ int sock_inet_get_dst(int fd, struct sockaddr *sa, socklen_t salen, int dir)
 
 #if defined(USE_TPROXY) && defined(SO_ORIGINAL_DST)
 		/* For TPROXY and Netfilter's NAT, we can retrieve the original
-		 * IPv4 address before DNAT/REDIRECT. However we must not do
-		 * that for sockets created using TPROXY (in transparent mode)
-		 * because sometimes users configure them with NOTRACK only in
-		 * one direction, leading to the connection being created anyway
-		 * in reverse direction, and even without this it seldom happens
-		 * that a late retransmit of a previous connection recreates the
-		 * conntrack entry.
+		 * IPv4 address before DNAT/REDIRECT. We must not do that with
+		 * other families because v6-mapped IPv4 addresses are still
+		 * reported as v4.
 		 */
-# if defined(IP_TRANSPARENT)
-		len = sizeof(val);
-		if (getsockopt(fd, IPPROTO_IP, IP_TRANSPARENT, &val, &len) == 0 && val)
-			return ret;
-# endif
 		if (getsockopt(fd, IPPROTO_IP, SO_ORIGINAL_DST, sa, &salen) == 0)
 			return 0;
 #endif
@@ -323,6 +315,12 @@ int sock_inet_bind_receiver(struct receiver *rx, char **errmsg)
 			err |= ERR_RETRYABLE;
 			goto bind_ret_err;
 		}
+		/* taking the other one's FD will result in it being marked
+		 * extern and being dup()ed. Let's mark the receiver as
+		 * inherited so that it properly bypasses all second-stage
+		 * setup and avoids being passed to new processes.
+		 */
+		rx->flags |= RX_F_INHERITED;
 		rx->fd = rx->shard_info->ref->fd;
 	}
 
@@ -457,7 +455,7 @@ int sock_inet_bind_receiver(struct receiver *rx, char **errmsg)
 	fd_insert(fd, rx->owner, rx->iocb, rx->bind_tgroup, rx->bind_thread);
 
 	/* for now, all regularly bound TCP listeners are exportable */
-	if (!(rx->flags & (RX_F_INHERITED_FD|RX_F_INHERITED_SOCK)))
+	if (!(rx->flags & RX_F_INHERITED))
 		HA_ATOMIC_OR(&fdtab[fd].state, FD_EXPORTED);
 
  bind_return:
@@ -475,6 +473,45 @@ int sock_inet_bind_receiver(struct receiver *rx, char **errmsg)
 	goto bind_return;
 }
 
+
+/* Detects IPv6 reachability: for this we perform a UDP connect to address
+ * 2001:: on port 53. No packet will be sent, it will just check the routing
+ * table towards this prefix for the majority of public addresses. In case of
+ * error we assume no IPv6 connectivity.
+ *
+ * Returns non-zero if inet6 looks reachable, otherwise zero. This considers
+ * the last result if it ages less than 30s, otherwise triggers a new test
+ * which updates <sock_inet6_seems_reachable> and <last_inet6_check>.
+ */
+int is_inet6_reachable(void)
+{
+	uint last_check = HA_ATOMIC_LOAD(&last_inet6_check);
+	struct sockaddr_in6 dest = { };
+	int ret = 0;
+	int fd;
+
+	if (tick_isset(last_check) &&
+	    !tick_is_expired(tick_add(last_check, INET6_CONNECTIVITY_CACHE_TIME), HA_ATOMIC_LOAD(&global_now_ms)))
+		return HA_ATOMIC_LOAD(&sock_inet6_seems_reachable);
+
+	/* update the test date to ensure nobody else does it in parallel */
+	HA_ATOMIC_STORE(&last_inet6_check, HA_ATOMIC_LOAD(&global_now_ms));
+
+	fd = socket(AF_INET6, SOCK_DGRAM, 0);
+	if (fd >= 0) {
+		dest.sin6_family = AF_INET6;
+		dest.sin6_addr.s6_addr[0] = 0x20;
+		dest.sin6_addr.s6_addr[1] = 0x01;
+		dest.sin6_port = htons(53);
+		if (connect(fd, (struct sockaddr*)&dest, sizeof(dest)) == 0)
+			ret = 1;
+		close(fd);
+	}
+
+	HA_ATOMIC_STORE(&sock_inet6_seems_reachable, ret);
+	return ret;
+}
+
 static void sock_inet_prepare()
 {
 	int fd, val;
