@@ -60,6 +60,7 @@
 #include <haproxy/stats-proxy.h>
 #include <haproxy/stconn.h>
 #include <haproxy/stream.h>
+#include <haproxy/stress.h>
 #include <haproxy/task.h>
 #include <haproxy/ticks.h>
 #include <haproxy/time.h>
@@ -91,86 +92,86 @@ const char *stat_status_codes[STAT_STATUS_SIZE] = {
  * to always use the exact same name except that the strings for new names must
  * be lower case or CamelCase while the enum entries must be upper case.
  */
-const struct name_desc stat_cols_info[ST_I_INF_MAX] = {
-	[ST_I_INF_NAME]                           = { .name = "Name",                        .desc = "Product name" },
-	[ST_I_INF_VERSION]                        = { .name = "Version",                     .desc = "Product version" },
-	[ST_I_INF_RELEASE_DATE]                   = { .name = "Release_date",                .desc = "Date of latest source code update" },
-	[ST_I_INF_NBTHREAD]                       = { .name = "Nbthread",                    .desc = "Number of started threads (global.nbthread)" },
-	[ST_I_INF_NBPROC]                         = { .name = "Nbproc",                      .desc = "Number of started worker processes (historical, always 1)" },
-	[ST_I_INF_PROCESS_NUM]                    = { .name = "Process_num",                 .desc = "Relative worker process number (1)" },
-	[ST_I_INF_PID]                            = { .name = "Pid",                         .desc = "This worker process identifier for the system" },
-	[ST_I_INF_UPTIME]                         = { .name = "Uptime",                      .desc = "How long ago this worker process was started (days+hours+minutes+seconds)" },
-	[ST_I_INF_UPTIME_SEC]                     = { .name = "Uptime_sec",                  .desc = "How long ago this worker process was started (seconds)" },
-	[ST_I_INF_START_TIME_SEC]                 = { .name = "Start_time_sec",              .desc = "Start time in seconds" },
-	[ST_I_INF_MEMMAX_MB]                      = { .name = "Memmax_MB",                   .desc = "Worker process's hard limit on memory usage in MB (-m on command line)" },
-	[ST_I_INF_MEMMAX_BYTES]                   = { .name = "Memmax_bytes",                .desc = "Worker process's hard limit on memory usage in byes (-m on command line)" },
-	[ST_I_INF_POOL_ALLOC_MB]                  = { .name = "PoolAlloc_MB",                .desc = "Amount of memory allocated in pools (in MB)" },
-	[ST_I_INF_POOL_ALLOC_BYTES]               = { .name = "PoolAlloc_bytes",             .desc = "Amount of memory allocated in pools (in bytes)" },
-	[ST_I_INF_POOL_USED_MB]                   = { .name = "PoolUsed_MB",                 .desc = "Amount of pool memory currently used (in MB)" },
-	[ST_I_INF_POOL_USED_BYTES]                = { .name = "PoolUsed_bytes",              .desc = "Amount of pool memory currently used (in bytes)" },
-	[ST_I_INF_POOL_FAILED]                    = { .name = "PoolFailed",                  .desc = "Number of failed pool allocations since this worker was started" },
-	[ST_I_INF_ULIMIT_N]                       = { .name = "Ulimit-n",                    .desc = "Hard limit on the number of per-process file descriptors" },
-	[ST_I_INF_MAXSOCK]                        = { .name = "Maxsock",                     .desc = "Hard limit on the number of per-process sockets" },
-	[ST_I_INF_MAXCONN]                        = { .name = "Maxconn",                     .desc = "Hard limit on the number of per-process connections (configured or imposed by Ulimit-n)" },
-	[ST_I_INF_HARD_MAXCONN]                   = { .name = "Hard_maxconn",                .desc = "Hard limit on the number of per-process connections (imposed by Memmax_MB or Ulimit-n)" },
-	[ST_I_INF_CURR_CONN]                      = { .name = "CurrConns",                   .desc = "Current number of connections on this worker process" },
-	[ST_I_INF_CUM_CONN]                       = { .name = "CumConns",                    .desc = "Total number of connections on this worker process since started" },
-	[ST_I_INF_CUM_REQ]                        = { .name = "CumReq",                      .desc = "Total number of requests on this worker process since started" },
-	[ST_I_INF_MAX_SSL_CONNS]                  = { .name = "MaxSslConns",                 .desc = "Hard limit on the number of per-process SSL endpoints (front+back), 0=unlimited" },
-	[ST_I_INF_CURR_SSL_CONNS]                 = { .name = "CurrSslConns",                .desc = "Current number of SSL endpoints on this worker process (front+back)" },
-	[ST_I_INF_CUM_SSL_CONNS]                  = { .name = "CumSslConns",                 .desc = "Total number of SSL endpoints on this worker process since started (front+back)" },
-	[ST_I_INF_MAXPIPES]                       = { .name = "Maxpipes",                    .desc = "Hard limit on the number of pipes for splicing, 0=unlimited" },
-	[ST_I_INF_PIPES_USED]                     = { .name = "PipesUsed",                   .desc = "Current number of pipes in use in this worker process" },
-	[ST_I_INF_PIPES_FREE]                     = { .name = "PipesFree",                   .desc = "Current number of allocated and available pipes in this worker process" },
-	[ST_I_INF_CONN_RATE]                      = { .name = "ConnRate",                    .desc = "Number of front connections created on this worker process over the last second" },
-	[ST_I_INF_CONN_RATE_LIMIT]                = { .name = "ConnRateLimit",               .desc = "Hard limit for ConnRate (global.maxconnrate)" },
-	[ST_I_INF_MAX_CONN_RATE]                  = { .name = "MaxConnRate",                 .desc = "Highest ConnRate reached on this worker process since started (in connections per second)" },
-	[ST_I_INF_SESS_RATE]                      = { .name = "SessRate",                    .desc = "Number of sessions created on this worker process over the last second" },
-	[ST_I_INF_SESS_RATE_LIMIT]                = { .name = "SessRateLimit",               .desc = "Hard limit for SessRate (global.maxsessrate)" },
-	[ST_I_INF_MAX_SESS_RATE]                  = { .name = "MaxSessRate",                 .desc = "Highest SessRate reached on this worker process since started (in sessions per second)" },
-	[ST_I_INF_SSL_RATE]                       = { .name = "SslRate",                     .desc = "Number of SSL connections created on this worker process over the last second" },
-	[ST_I_INF_SSL_RATE_LIMIT]                 = { .name = "SslRateLimit",                .desc = "Hard limit for SslRate (global.maxsslrate)" },
-	[ST_I_INF_MAX_SSL_RATE]                   = { .name = "MaxSslRate",                  .desc = "Highest SslRate reached on this worker process since started (in connections per second)" },
-	[ST_I_INF_SSL_FRONTEND_KEY_RATE]          = { .name = "SslFrontendKeyRate",          .desc = "Number of SSL keys created on frontends in this worker process over the last second" },
-	[ST_I_INF_SSL_FRONTEND_MAX_KEY_RATE]      = { .name = "SslFrontendMaxKeyRate",       .desc = "Highest SslFrontendKeyRate reached on this worker process since started (in SSL keys per second)" },
-	[ST_I_INF_SSL_FRONTEND_SESSION_REUSE_PCT] = { .name = "SslFrontendSessionReuse_pct", .desc = "Percent of frontend SSL connections which did not require a new key" },
-	[ST_I_INF_SSL_BACKEND_KEY_RATE]           = { .name = "SslBackendKeyRate",           .desc = "Number of SSL keys created on backends in this worker process over the last second" },
-	[ST_I_INF_SSL_BACKEND_MAX_KEY_RATE]       = { .name = "SslBackendMaxKeyRate",        .desc = "Highest SslBackendKeyRate reached on this worker process since started (in SSL keys per second)" },
-	[ST_I_INF_SSL_CACHE_LOOKUPS]              = { .name = "SslCacheLookups",             .desc = "Total number of SSL session ID lookups in the SSL session cache on this worker since started" },
-	[ST_I_INF_SSL_CACHE_MISSES]               = { .name = "SslCacheMisses",              .desc = "Total number of SSL session ID lookups that didn't find a session in the SSL session cache on this worker since started" },
-	[ST_I_INF_COMPRESS_BPS_IN]                = { .name = "CompressBpsIn",               .desc = "Number of bytes submitted to the HTTP compressor in this worker process over the last second" },
-	[ST_I_INF_COMPRESS_BPS_OUT]               = { .name = "CompressBpsOut",              .desc = "Number of bytes emitted by the HTTP compressor in this worker process over the last second" },
-	[ST_I_INF_COMPRESS_BPS_RATE_LIM]          = { .name = "CompressBpsRateLim",          .desc = "Limit of CompressBpsOut beyond which HTTP compression is automatically disabled" },
-	[ST_I_INF_ZLIB_MEM_USAGE]                 = { .name = "ZlibMemUsage",                .desc = "Amount of memory currently used by HTTP compression on the current worker process (in bytes)" },
-	[ST_I_INF_MAX_ZLIB_MEM_USAGE]             = { .name = "MaxZlibMemUsage",             .desc = "Limit on the amount of memory used by HTTP compression above which it is automatically disabled (in bytes, see global.maxzlibmem)" },
-	[ST_I_INF_TASKS]                          = { .name = "Tasks",                       .desc = "Total number of tasks in the current worker process (active + sleeping)" },
-	[ST_I_INF_RUN_QUEUE]                      = { .name = "Run_queue",                   .desc = "Total number of active tasks+tasklets in the current worker process" },
-	[ST_I_INF_IDLE_PCT]                       = { .name = "Idle_pct",                    .desc = "Percentage of last second spent waiting in the current worker thread" },
-	[ST_I_INF_NODE]                           = { .name = "node",                        .desc = "Node name (global.node)" },
-	[ST_I_INF_DESCRIPTION]                    = { .name = "description",                 .desc = "Node description (global.description)" },
-	[ST_I_INF_STOPPING]                       = { .name = "Stopping",                    .desc = "1 if the worker process is currently stopping, otherwise zero" },
-	[ST_I_INF_JOBS]                           = { .name = "Jobs",                        .desc = "Current number of active jobs on the current worker process (frontend connections, master connections, listeners)" },
-	[ST_I_INF_UNSTOPPABLE_JOBS]               = { .name = "Unstoppable Jobs",            .desc = "Current number of unstoppable jobs on the current worker process (master connections)" },
-	[ST_I_INF_LISTENERS]                      = { .name = "Listeners",                   .desc = "Current number of active listeners on the current worker process" },
-	[ST_I_INF_ACTIVE_PEERS]                   = { .name = "ActivePeers",                 .desc = "Current number of verified active peers connections on the current worker process" },
-	[ST_I_INF_CONNECTED_PEERS]                = { .name = "ConnectedPeers",              .desc = "Current number of peers having passed the connection step on the current worker process" },
-	[ST_I_INF_DROPPED_LOGS]                   = { .name = "DroppedLogs",                 .desc = "Total number of dropped logs for current worker process since started" },
-	[ST_I_INF_BUSY_POLLING]                   = { .name = "BusyPolling",                 .desc = "1 if busy-polling is currently in use on the worker process, otherwise zero (config.busy-polling)" },
-	[ST_I_INF_FAILED_RESOLUTIONS]             = { .name = "FailedResolutions",           .desc = "Total number of failed DNS resolutions in current worker process since started" },
-	[ST_I_INF_TOTAL_BYTES_OUT]                = { .name = "TotalBytesOut",               .desc = "Total number of bytes emitted by current worker process since started" },
-	[ST_I_INF_TOTAL_SPLICED_BYTES_OUT]        = { .name = "TotalSplicedBytesOut",        .desc = "Total number of bytes emitted by current worker process through a kernel pipe since started" },
-	[ST_I_INF_BYTES_OUT_RATE]                 = { .name = "BytesOutRate",                .desc = "Number of bytes emitted by current worker process over the last second" },
-	[ST_I_INF_DEBUG_COMMANDS_ISSUED]          = { .name = "DebugCommandsIssued",         .desc = "Number of debug commands issued on this process (anything > 0 is unsafe)" },
-	[ST_I_INF_CUM_LOG_MSGS]                   = { .name = "CumRecvLogs",                 .desc = "Total number of log messages received by log-forwarding listeners on this worker process since started" },
-	[ST_I_INF_BUILD_INFO]                     = { .name = "Build info",                  .desc = "Build info" },
-	[ST_I_INF_TAINTED]                        = { .name = "Tainted",                     .desc = "Experimental features used" },
-	[ST_I_INF_WARNINGS]                       = { .name = "TotalWarnings",               .desc = "Total warnings issued" },
-	[ST_I_INF_MAXCONN_REACHED]                = { .name = "MaxconnReached",              .desc = "Number of times an accepted connection resulted in Maxconn being reached" },
-	[ST_I_INF_BOOTTIME_MS]                    = { .name = "BootTime_ms",                 .desc = "How long ago it took to parse and process the config before being ready (milliseconds)" },
-	[ST_I_INF_NICED_TASKS]                    = { .name = "Niced_tasks",                 .desc = "Total number of active tasks+tasklets in the current worker process (Run_queue) that are niced" },
-	[ST_I_INF_CURR_STRM]                      = { .name = "CurrStreams",                 .desc = "Current number of streams on this worker process" },
-	[ST_I_INF_CUM_STRM]                       = { .name = "CumStreams",                  .desc = "Total number of streams created on this worker process since started" },
-	[ST_I_INF_WARN_BLOCKED]                   = { .name = "BlockedTrafficWarnings",      .desc = "Total number of warnings issued about traffic being blocked by too slow a task" },
+const struct stat_col stat_cols_info[ST_I_INF_MAX] = {
+	[ST_I_INF_NAME]                           = { .name = "Name",                        .alt_name = NULL,                            .desc = "Product name" },
+	[ST_I_INF_VERSION]                        = { .name = "Version",                     .alt_name = NULL,                            .desc = "Product version" },
+	[ST_I_INF_RELEASE_DATE]                   = { .name = "Release_date",                .alt_name = NULL,                            .desc = "Date of latest source code update" },
+	[ST_I_INF_NBTHREAD]                       = { .name = "Nbthread",                    .alt_name = "nbthread",                      .desc = "Number of started threads (global.nbthread)" },
+	[ST_I_INF_NBPROC]                         = { .name = "Nbproc",                      .alt_name = "nbproc",                        .desc = "Number of started worker processes (historical, always 1)" },
+	[ST_I_INF_PROCESS_NUM]                    = { .name = "Process_num",                 .alt_name = "relative_process_id",           .desc = "Relative worker process number (1)", },
+	[ST_I_INF_PID]                            = { .name = "Pid",                         .alt_name = NULL,                            .desc = "This worker process identifier for the system" },
+	[ST_I_INF_UPTIME]                         = { .name = "Uptime",                      .alt_name = NULL,                            .desc = "How long ago this worker process was started (days+hours+minutes+seconds)" },
+	[ST_I_INF_UPTIME_SEC]                     = { .name = "Uptime_sec",                  .alt_name = "uptime_seconds",                .desc = "How long ago this worker process was started (seconds)" },
+	[ST_I_INF_START_TIME_SEC]                 = { .name = "Start_time_sec",              .alt_name = "start_time_seconds",            .desc = "Start time in seconds" },
+	[ST_I_INF_MEMMAX_MB]                      = { .name = "Memmax_MB",                   .alt_name = NULL,                            .desc = "Worker process's hard limit on memory usage in MB (-m on command line)" },
+	[ST_I_INF_MEMMAX_BYTES]                   = { .name = "Memmax_bytes",                .alt_name = "max_memory_bytes",              .desc = "Worker process's hard limit on memory usage in byes (-m on command line)" },
+	[ST_I_INF_POOL_ALLOC_MB]                  = { .name = "PoolAlloc_MB",                .alt_name = NULL,                            .desc = "Amount of memory allocated in pools (in MB)" },
+	[ST_I_INF_POOL_ALLOC_BYTES]               = { .name = "PoolAlloc_bytes",             .alt_name = "pool_allocated_bytes",          .desc = "Amount of memory allocated in pools (in bytes)" },
+	[ST_I_INF_POOL_USED_MB]                   = { .name = "PoolUsed_MB",                 .alt_name = NULL,                            .desc = "Amount of pool memory currently used (in MB)" },
+	[ST_I_INF_POOL_USED_BYTES]                = { .name = "PoolUsed_bytes",              .alt_name = "pool_used_bytes",               .desc = "Amount of pool memory currently used (in bytes)" },
+	[ST_I_INF_POOL_FAILED]                    = { .name = "PoolFailed",                  .alt_name = "pool_failures_total",           .desc = "Number of failed pool allocations since this worker was started" },
+	[ST_I_INF_ULIMIT_N]                       = { .name = "Ulimit-n",                    .alt_name = "max_fds",                       .desc = "Hard limit on the number of per-process file descriptors" },
+	[ST_I_INF_MAXSOCK]                        = { .name = "Maxsock",                     .alt_name = "max_sockets",                   .desc = "Hard limit on the number of per-process sockets", },
+	[ST_I_INF_MAXCONN]                        = { .name = "Maxconn",                     .alt_name = "max_connections",               .desc = "Hard limit on the number of per-process connections (configured or imposed by Ulimit-n)" },
+	[ST_I_INF_HARD_MAXCONN]                   = { .name = "Hard_maxconn",                .alt_name = "hard_max_connections",          .desc = "Hard limit on the number of per-process connections (imposed by Memmax_MB or Ulimit-n)" },
+	[ST_I_INF_CURR_CONN]                      = { .name = "CurrConns",                   .alt_name = "current_connections",           .desc = "Current number of connections on this worker process" },
+	[ST_I_INF_CUM_CONN]                       = { .name = "CumConns",                    .alt_name = "connections_total",             .desc = "Total number of connections on this worker process since started" },
+	[ST_I_INF_CUM_REQ]                        = { .name = "CumReq",                      .alt_name = "requests_total",                .desc = "Total number of requests on this worker process since started" },
+	[ST_I_INF_MAX_SSL_CONNS]                  = { .name = "MaxSslConns",                 .alt_name = "max_ssl_connections",           .desc = "Hard limit on the number of per-process SSL endpoints (front+back), 0=unlimited" },
+	[ST_I_INF_CURR_SSL_CONNS]                 = { .name = "CurrSslConns",                .alt_name = "current_ssl_connections",       .desc = "Current number of SSL endpoints on this worker process (front+back)" },
+	[ST_I_INF_CUM_SSL_CONNS]                  = { .name = "CumSslConns",                 .alt_name = "ssl_connections_total",         .desc = "Total number of SSL endpoints on this worker process since started (front+back)" },
+	[ST_I_INF_MAXPIPES]                       = { .name = "Maxpipes",                    .alt_name = "max_pipes",                     .desc = "Hard limit on the number of pipes for splicing, 0=unlimited" },
+	[ST_I_INF_PIPES_USED]                     = { .name = "PipesUsed",                   .alt_name = "pipes_used_total",              .desc = "Current number of pipes in use in this worker process" },
+	[ST_I_INF_PIPES_FREE]                     = { .name = "PipesFree",                   .alt_name = "pipes_free_total",              .desc = "Current number of allocated and available pipes in this worker process" },
+	[ST_I_INF_CONN_RATE]                      = { .name = "ConnRate",                    .alt_name = "current_connection_rate",       .desc = "Number of front connections created on this worker process over the last second" },
+	[ST_I_INF_CONN_RATE_LIMIT]                = { .name = "ConnRateLimit",               .alt_name = "limit_connection_rate",         .desc = "Hard limit for ConnRate (global.maxconnrate)", },
+	[ST_I_INF_MAX_CONN_RATE]                  = { .name = "MaxConnRate",                 .alt_name = "max_connection_rate",           .desc = "Highest ConnRate reached on this worker process since started (in connections per second)" },
+	[ST_I_INF_SESS_RATE]                      = { .name = "SessRate",                    .alt_name = "current_session_rate",          .desc = "Number of sessions created on this worker process over the last second" },
+	[ST_I_INF_SESS_RATE_LIMIT]                = { .name = "SessRateLimit",               .alt_name = "limit_session_rate",            .desc = "Hard limit for SessRate (global.maxsessrate)" },
+	[ST_I_INF_MAX_SESS_RATE]                  = { .name = "MaxSessRate",                 .alt_name = "max_session_rate",              .desc = "Highest SessRate reached on this worker process since started (in sessions per second)" },
+	[ST_I_INF_SSL_RATE]                       = { .name = "SslRate",                     .alt_name = "current_ssl_rate",              .desc = "Number of SSL connections created on this worker process over the last second" },
+	[ST_I_INF_SSL_RATE_LIMIT]                 = { .name = "SslRateLimit",                .alt_name = "limit_ssl_rate",                .desc = "Hard limit for SslRate (global.maxsslrate)" },
+	[ST_I_INF_MAX_SSL_RATE]                   = { .name = "MaxSslRate",                  .alt_name = "max_ssl_rate",                  .desc = "Highest SslRate reached on this worker process since started (in connections per second)" },
+	[ST_I_INF_SSL_FRONTEND_KEY_RATE]          = { .name = "SslFrontendKeyRate",          .alt_name = "current_frontend_ssl_key_rate", .desc = "Number of SSL keys created on frontends in this worker process over the last second", },
+	[ST_I_INF_SSL_FRONTEND_MAX_KEY_RATE]      = { .name = "SslFrontendMaxKeyRate",       .alt_name = "max_frontend_ssl_key_rate",     .desc = "Highest SslFrontendKeyRate reached on this worker process since started (in SSL keys per second)", },
+	[ST_I_INF_SSL_FRONTEND_SESSION_REUSE_PCT] = { .name = "SslFrontendSessionReuse_pct", .alt_name = "frontend_ssl_reuse",            .desc = "Percent of frontend SSL connections which did not require a new key" },
+	[ST_I_INF_SSL_BACKEND_KEY_RATE]           = { .name = "SslBackendKeyRate",           .alt_name = "current_backend_ssl_key_rate",  .desc = "Number of SSL keys created on backends in this worker process over the last second" },
+	[ST_I_INF_SSL_BACKEND_MAX_KEY_RATE]       = { .name = "SslBackendMaxKeyRate",        .alt_name = "max_backend_ssl_key_rate",      .desc = "Highest SslBackendKeyRate reached on this worker process since started (in SSL keys per second)" },
+	[ST_I_INF_SSL_CACHE_LOOKUPS]              = { .name = "SslCacheLookups",             .alt_name = "ssl_cache_lookups_total",       .desc = "Total number of SSL session ID lookups in the SSL session cache on this worker since started" },
+	[ST_I_INF_SSL_CACHE_MISSES]               = { .name = "SslCacheMisses",              .alt_name = "ssl_cache_misses_total",        .desc = "Total number of SSL session ID lookups that didn't find a session in the SSL session cache on this worker since started" },
+	[ST_I_INF_COMPRESS_BPS_IN]                = { .name = "CompressBpsIn",               .alt_name = "http_comp_bytes_in_total",      .desc = "Number of bytes submitted to the HTTP compressor in this worker process over the last second" },
+	[ST_I_INF_COMPRESS_BPS_OUT]               = { .name = "CompressBpsOut",              .alt_name = "http_comp_bytes_out_total",     .desc = "Number of bytes emitted by the HTTP compressor in this worker process over the last second" },
+	[ST_I_INF_COMPRESS_BPS_RATE_LIM]          = { .name = "CompressBpsRateLim",          .alt_name = "limit_http_comp",               .desc = "Limit of CompressBpsOut beyond which HTTP compression is automatically disabled" },
+	[ST_I_INF_ZLIB_MEM_USAGE]                 = { .name = "ZlibMemUsage",                .alt_name = "current_zlib_memory",           .desc = "Amount of memory currently used by HTTP compression on the current worker process (in bytes)" },
+	[ST_I_INF_MAX_ZLIB_MEM_USAGE]             = { .name = "MaxZlibMemUsage",             .alt_name = "max_zlib_memory",               .desc = "Limit on the amount of memory used by HTTP compression above which it is automatically disabled (in bytes, see global.maxzlibmem)" },
+	[ST_I_INF_TASKS]                          = { .name = "Tasks",                       .alt_name = "current_tasks",                 .desc = "Total number of tasks in the current worker process (active + sleeping)" },
+	[ST_I_INF_RUN_QUEUE]                      = { .name = "Run_queue",                   .alt_name = "current_run_queue",             .desc = "Total number of active tasks+tasklets in the current worker process" },
+	[ST_I_INF_IDLE_PCT]                       = { .name = "Idle_pct",                    .alt_name = "idle_time_percent",             .desc = "Percentage of last second spent waiting in the current worker thread" },
+	[ST_I_INF_NODE]                           = { .name = "node",                        .alt_name = "node",                          .desc = "Node name (global.node)" },
+	[ST_I_INF_DESCRIPTION]                    = { .name = "description",                 .alt_name = "description",                   .desc = "Node description (global.description)" },
+	[ST_I_INF_STOPPING]                       = { .name = "Stopping",                    .alt_name = "stopping",                      .desc = "1 if the worker process is currently stopping, otherwise zero" },
+	[ST_I_INF_JOBS]                           = { .name = "Jobs",                        .alt_name = "jobs",                          .desc = "Current number of active jobs on the current worker process (frontend connections, master connections, listeners)" },
+	[ST_I_INF_UNSTOPPABLE_JOBS]               = { .name = "Unstoppable Jobs",            .alt_name = "unstoppable_jobs",              .desc = "Current number of unstoppable jobs on the current worker process (master connections)" },
+	[ST_I_INF_LISTENERS]                      = { .name = "Listeners",                   .alt_name = "listeners",                     .desc = "Current number of active listeners on the current worker process" },
+	[ST_I_INF_ACTIVE_PEERS]                   = { .name = "ActivePeers",                 .alt_name = "active_peers",                  .desc = "Current number of verified active peers connections on the current worker process" },
+	[ST_I_INF_CONNECTED_PEERS]                = { .name = "ConnectedPeers",              .alt_name = "connected_peers",               .desc = "Current number of peers having passed the connection step on the current worker process" },
+	[ST_I_INF_DROPPED_LOGS]                   = { .name = "DroppedLogs",                 .alt_name = "dropped_logs_total",            .desc = "Total number of dropped logs for current worker process since started" },
+	[ST_I_INF_BUSY_POLLING]                   = { .name = "BusyPolling",                 .alt_name = "busy_polling_enabled",          .desc = "1 if busy-polling is currently in use on the worker process, otherwise zero (config.busy-polling)" },
+	[ST_I_INF_FAILED_RESOLUTIONS]             = { .name = "FailedResolutions",           .alt_name = "failed_resolutions",            .desc = "Total number of failed DNS resolutions in current worker process since started" },
+	[ST_I_INF_TOTAL_BYTES_OUT]                = { .name = "TotalBytesOut",               .alt_name = "bytes_out_total",               .desc = "Total number of bytes emitted by current worker process since started" },
+	[ST_I_INF_TOTAL_SPLICED_BYTES_OUT]        = { .name = "TotalSplicedBytesOut",        .alt_name = "spliced_bytes_out_total",       .desc = "Total number of bytes emitted by current worker process through a kernel pipe since started" },
+	[ST_I_INF_BYTES_OUT_RATE]                 = { .name = "BytesOutRate",                .alt_name = "bytes_out_rate",                .desc = "Number of bytes emitted by current worker process over the last second" },
+	[ST_I_INF_DEBUG_COMMANDS_ISSUED]          = { .name = "DebugCommandsIssued",         .alt_name = NULL,                            .desc = "Number of debug commands issued on this process (anything > 0 is unsafe)" },
+	[ST_I_INF_CUM_LOG_MSGS]                   = { .name = "CumRecvLogs",                 .alt_name = "recv_logs_total",               .desc = "Total number of log messages received by log-forwarding listeners on this worker process since started" },
+	[ST_I_INF_BUILD_INFO]                     = { .name = "Build info",                  .alt_name = "build_info",                    .desc = "Build info" },
+	[ST_I_INF_TAINTED]                        = { .name = "Tainted",                     .alt_name = NULL,                            .desc = "Experimental features used" },
+	[ST_I_INF_WARNINGS]                       = { .name = "TotalWarnings",               .alt_name = "total_warnings",                .desc = "Total warnings issued" },
+	[ST_I_INF_MAXCONN_REACHED]                = { .name = "MaxconnReached",              .alt_name = NULL,                            .desc = "Number of times an accepted connection resulted in Maxconn being reached" },
+	[ST_I_INF_BOOTTIME_MS]                    = { .name = "BootTime_ms",                 .alt_name = NULL,                            .desc = "How long ago it took to parse and process the config before being ready (milliseconds)" },
+	[ST_I_INF_NICED_TASKS]                    = { .name = "Niced_tasks",                 .alt_name = NULL,                            .desc = "Total number of active tasks+tasklets in the current worker process (Run_queue) that are niced" },
+	[ST_I_INF_CURR_STRM]                      = { .name = "CurrStreams",                 .alt_name = NULL,                            .desc = "Current number of streams on this worker process" },
+	[ST_I_INF_CUM_STRM]                       = { .name = "CumStreams",                  .alt_name = NULL,                            .desc = "Total number of streams created on this worker process since started" },
+	[ST_I_INF_WARN_BLOCKED]                   = { .name = "BlockedTrafficWarnings",      .alt_name = NULL,                            .desc = "Total number of warnings issued about traffic being blocked by too slow a task" },
 };
 
 /* one line of info */
@@ -231,7 +232,8 @@ int stats_putchk(struct appctx *appctx, struct buffer *buf, struct htx *htx)
 	struct buffer *chk = &ctx->chunk;
 
 	if (htx) {
-		if (b_data(chk) > htx_free_data_space(htx)) {
+		if (STRESS_RUN1(!htx_is_empty(htx),
+		                b_data(chk) > htx_free_data_space(htx))) {
 			applet_fl_set(appctx, APPCTX_FL_OUTBLK_FULL);
 			return 0;
 		}
@@ -242,7 +244,7 @@ int stats_putchk(struct appctx *appctx, struct buffer *buf, struct htx *htx)
 		chunk_reset(chk);
 	}
 	else if (buf) {
-		if (b_data(chk) > b_room(buf)) {
+		if (STRESS_RUN1(b_data(buf), b_data(chk) > b_room(buf))) {
 			se_fl_set(appctx->sedesc, SE_FL_RCV_MORE | SE_FL_WANT_ROOM);
 			return 0;
 		}
@@ -250,29 +252,31 @@ int stats_putchk(struct appctx *appctx, struct buffer *buf, struct htx *htx)
 		chunk_reset(chk);
 	}
 	else {
-		if (applet_putchk(appctx, chk) == -1)
+		if (STRESS_RUN1(applet_putchk_stress(appctx, chk) == -1,
+		                applet_putchk(appctx, chk) == -1)) {
 			return 0;
+		}
 	}
 	return 1;
 }
 
-
 int stats_is_full(struct appctx *appctx, struct buffer *buf, struct htx *htx)
 {
 	if (htx) {
-		if (htx_almost_full(htx)) {
+		if (STRESS_RUN1(!htx_is_empty(htx), htx_almost_full(htx))) {
 			applet_fl_set(appctx, APPCTX_FL_OUTBLK_FULL);
 			goto full;
 		}
 	}
 	else if (buf) {
-		if (buffer_almost_full(buf)) {
+		if (STRESS_RUN1(b_data(buf), buffer_almost_full(buf))) {
 			se_fl_set(appctx->sedesc, SE_FL_RCV_MORE | SE_FL_WANT_ROOM);
 			goto full;
 		}
 	}
 	else {
-		if (buffer_almost_full(&appctx->outbuf))  {
+		if (STRESS_RUN1(b_data(&appctx->outbuf),
+		                buffer_almost_full(&appctx->outbuf))) {
 			applet_fl_set(appctx, APPCTX_FL_OUTBLK_FULL);
 			goto full;
 		}
@@ -289,9 +293,8 @@ const char *stats_scope_ptr(struct appctx *appctx)
 	struct htx_blk *blk;
 	struct ist uri;
 
-	blk = htx_get_head_blk(htx);
-	BUG_ON(!blk || htx_get_blk_type(blk) != HTX_BLK_REQ_SL);
-	ALREADY_CHECKED(blk);
+	blk = ASSUME_NONNULL(htx_get_head_blk(htx));
+	BUG_ON(htx_get_blk_type(blk) != HTX_BLK_REQ_SL);
 	uri = htx_sl_req_uri(htx_get_blk_ptr(htx, blk));
 	return uri.ptr + ctx->scope_str;
 }
